import java.util.HashMap;

import javax.management.InstanceOfQueryExp;

import org.emau.icmvc.mosaic.pl.dsa.extrans.pprocessor.modules.ModuleException;
import org.emau.icmvc.mosaic.pl.dsa.load.dtotrans.dto.*;

/**
 * This groovy-module is a example.
 * It is to show the possibilities of what and how is possible with groovy.
 * 
 * The task of the module is the birth year to increase by one if it is a male patient.
 * The optional parameter "over" may be a minimum age to be specified.
 * 
 * @author schuldtr
 * @version 0.1
 * @since 15.07.2014
 */
class PlusOneBirthyearAtMale
{
    ObservationGenericDTO process (ObservationGenericDTO dto)
    {
        return process(dto, new HashMap<String, Object>());
    }

    ObservationGenericDTO process (ObservationGenericDTO dto, HashMap<String, Object> parameters)
    {
        if (dto instanceof ObservationValueElementValueDTO) {
            Integer year = ((ObservationValueElementValueDTO) dto).getValue().toInteger();

            if (parameters.containsKey("over") && year <= Integer.parseInt(parameters.get("over"))) {
                // end
                return dto;
            }

            ObservationGenericDTO parent = dto.getParentElement();
            List<ObservationValueElementAbstractDTO> childs = null;

            // get childs
            if (parent instanceof ObservationValueElementAbstractDTO) {
                childs = ((ObservationValueElementAbstractDTO) parent).getChildValues();
            } else if (parent instanceof ObservationContainerDTO) {
                childs = ((ObservationContainerDTO) parent).getValues();
            }

            // find male patient and change birthyear to one up
            childs.any() { child ->
                if (child instanceof ObservationValueElementValueDTO
                    && child.getIdentifier() == "patient.gender"
                    && ((ObservationValueElementValueDTO) child).getValue().substring(0, 1).toLowerCase() == "m" 
                ) {
                    ((ObservationValueElementValueDTO) dto).setValue((year + 1).toString());
                    // break
                    return true;
                }
            }
        } else {
            throw new ModuleException("unsupported dto-type: " + dto.getClass().getName());
        }

        // end
        return dto;
    }
}