#!/bin/bash
# ###license-information-start###
# The MOSAIC Project - Toolbox for research
# __
# Copyright (C) 2009 - 2017 The MOSAIC Project - Institute for Community Medicine 
# University Medicine of Greifswald - mosaic@uni-greifswald.de
# __
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ###license-information-end###
# author: arne.blumentritt@uni-greifswald.de

version="1.2.4"
set -e
mkdir -p logs
chmod 777 logs
:> logs/install_out.log
:> logs/install_error.log
echo "Installation start time: $(date)" >> logs/install_out.log
clear

echo "~ The MOSAIC Project - Toolbox $version ~"
echo ""
echo "==================="
echo "=     Docker      ="
echo "==================="
echo "- Install Docker"

instDocker=1
if [ $(which docker) ]; then
	echo "- Docker is already installed."
	read -p "- Do you want to reinstall? [y/n] " -n 1 -r
	if [[ ! $REPLY =~ ^[Yy]$ ]]; then
		instDocker=0
	fi
fi

if [ $instDocker = 1 ]; then
	echo ""
	echo "- Please make sure you are connected to the internet."
	read -p "- Press [ENTER] to start: "
	echo "- Install Docker"
	#wget -qO- https://get.docker.com/ | sed 's/lxc-docker/lxc-docker-1.6.2/' | sh &>> logs/install_out.log
	apt-get update >> logs/install_out.log
	apt-get --yes --force-yes install docker-engine
fi

clear
echo "~ The MOSAIC Project - Toolbox $version ~"
echo ""
echo "==================="
echo "=     Docker      ="
echo "==================="
echo "- Install Docker"

read -e -p "- Set storage-path on host for config- and database-files: " -i "/opt" HOSTDIR
if [ ! -d "$HOSTDIR" ]; then
	echo "Given hostpath does not exists '$HOSTDIR'"
	exit 1;
fi
HOSTDIR=${HOSTDIR%/}

chmod +x -R **/*.sh

chmod 755 ./install_oc.sh
chmod 755 ./install_ttp.sh
chmod 755 ./install_portal.sh
chmod 755 ./install_ocdi.sh
#chmod 755 ./ocdi/init_ocdi.sh
chmod 755 ./uninstall.sh
chmod 755 ./start.sh
chmod 755 ./set_study_and_event.sh
chmod 755 ./set_passwords.sh

if [ -s ./logs/install_error.log ]; then
	echo "Installation of docker itself failed"
else
	./install_oc.sh $HOSTDIR 2>> logs/install_error.log
	if [ -s ./logs/install_error.log ]; then
		echo "Installation of container oc failed"
	else
		./install_ttp.sh $HOSTDIR 2>> logs/install_error.log
		if [ -s ./logs/install_error.log ]; then
			echo "Installation of container ttp failed"
		else
			./install_portal.sh $HOSTDIR 2>> logs/install_error.log
			if [ -s ./logs/install_error.log ]; then
				echo "Installation of container portal failed"
			else
				./install_ocdi.sh $HOSTDIR 2>> logs/install_error.log
				if [ -s ./logs/install_error.log ]; then
					echo "Installation of container ocdi failed"
				fi
			fi
		fi
	fi
fi


echo ""
chmod -R 777 $HOSTDIR

echo "installing open clinica bugfix for https"
docker cp ./oc/bugfix/jsp oc:/tomcat/webapps/OpenClinica/WEB-INF/ 2>> logs/install_error.log

if [ -s ./logs/install_error.log ]
then
	echo "==================="
	echo "=      ERROR      ="
	echo "==================="
	echo "- ERRORS occured during installation. Check logs/install_error.log."
else
	echo "==================="
	echo "=    COMPLETE     ="
	echo "==================="
	echo "- Thank you for using Mosaic Tools."
fi
echo ""
echo "Installation end time: $(date)" >> logs/install_out.log