#!/bin/bash
# ###license-information-start###
# The MOSAIC Project - Toolbox for research
# __
# Copyright (C) 2009 - 2017 The MOSAIC Project - Institute for Community Medicine 
# University Medicine of Greifswald - mosaic@uni-greifswald.de
# __
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ###license-information-end###
# author: arne.blumentritt@uni-greifswald.de

echo ""
echo "=========================="
echo "=   OpenClinica v.3.4.1  ="
echo "=========================="
echo ""


HOSTDIR=$1
if [ -z "$HOSTDIR" ]; then
	read -e -p "- Set hostpath for config- and database-files: " -i "/opt" HOSTDIR
	if [ ! -d "$HOSTDIR" ]; then
		echo "Given hostpath does not exists '$HOSTDIR'"
		exit 1;
	fi
fi
HOSTDIR=${HOSTDIR%/}

echo "- Install Postgres"
mkdir -p $HOSTDIR/openclinica
mkdir -p $HOSTDIR/openclinica/data
mkdir -p $HOSTDIR/openclinica/export
chmod -R 777 $HOSTDIR/openclinica

docker pull postgres:8 >> logs/install_out.log 2>> logs/install_error.log
docker run --name=ocdb-data -v $HOSTDIR/openclinica/data/:/var/lib/postgresql/data postgres:8 true >> logs/install_out.log 2>> logs/install_error.log

echo "- Start Postgres"
docker run --name=ocdb -d --volumes-from ocdb-data -e POSTGRES_PASSWORD=postgres123 -p 5432:5432 postgres:8 >> logs/install_out.log 2>> logs/install_error.log
sleep 20

echo "- Configure Postgres"
docker exec ocdb su postgres -c $'psql -c "CREATE ROLE clinica LOGIN ENCRYPTED PASSWORD \'clinica\' SUPERUSER NOINHERIT NOCREATEDB NOCREATEROLE" && psql -c "CREATE DATABASE openclinica WITH ENCODING=\'UTF8\' OWNER=clinica" && echo "host all clinica 0.0.0.0/0 md5" >> $PGDATA/pg_hba.conf && /usr/lib/postgresql/$PG_MAJOR/bin/pg_ctl reload -D $PGDATA' >> logs/install_out.log 2>> logs/install_error.log

echo "- Install OpenClinica"
docker pull piegsaj/openclinica:OC-3.4.1 >> logs/install_out.log 2>> logs/install_error.log
docker run --name=oc-data -v /tomcat/openclinica.data -v $HOSTDIR/openclinica/export/:/tomcat/openclinica.data/datasets piegsaj/openclinica:OC-3.4.1 true >> logs/install_out.log 2>> logs/install_error.log

echo "- Start OpenClinica"
docker run --name=oc -h oc -d --volumes-from oc-data -e TOMCAT_PASS="admin" -e LOG_LEVEL=INFO -e TZ=UTC-1 --link=ocdb:ocdb piegsaj/openclinica:OC-3.4.1 >> logs/install_out.log 2>> logs/install_error.log
docker stop oc >> logs/install_out.log 2>> logs/install_error.log
docker cp ./oc/server.xml oc:/tomcat/conf/server.xml
docker start oc >> logs/install_out.log 2>> logs/install_error.log
sleep 120

echo "- Configure OpenClinica"
docker exec ocdb su postgres -c $'psql -d openclinica -c "INSERT INTO public.user_account (user_name, passwd, active_study, status_id, owner_id, user_type_id, update_id, enabled, account_non_locked, lock_counter, run_webservices) VALUES (\'nvr_dispatcher\', \'701aae6a5f8dca50540c936c9e74c3a985687d33\', 1, 1, 1, 3, 2, true, true, 0, true);"' >> logs/install_out.log 2>> logs/install_error.log
sleep 1
docker exec ocdb su postgres -c $'psql -d openclinica -c "INSERT INTO public.study_user_role (role_name, study_id, status_id, owner_id, user_name) VALUES (\'coordinator\', 1, 1, 1, \'nvr_dispatcher\');"' >> logs/install_out.log 2>> logs/install_error.log
sleep 1
docker exec ocdb su postgres -c $'psql -d openclinica -c "INSERT INTO public.authorities (username, authority, version) VALUES (\'nvr_dispatcher\', \'ROLE_USER\', 0);"' >> logs/install_out.log 2>> logs/install_error.log