#!/bin/bash
# ###license-information-start###
# The MOSAIC Project - Toolbox for research
# __
# Copyright (C) 2009 - 2017 The MOSAIC Project - Institute for Community Medicine
# University Medicine of Greifswald - mosaic@uni-greifswald.de
# __
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ###license-information-end###
# author: ronny.schuldt@uni-greifswald.de
##############################################################################

echo ""
echo "=========================="
echo "=      OCDI v.1.0.10      ="
echo "=========================="
echo ""
echo "Changes: Optimized JVM Memory Allocation"
echo ""

HOSTDIR=$1
if [ -z "$HOSTDIR" ]; then
	read -e -p "- Set storage-path on host for config- and database-files: " -i "/opt" HOSTDIR
	if [ ! -d "$HOSTDIR" ]; then
		echo "Given hostpath does not exists '$HOSTDIR'"
		exit 1;
	fi
fi
HOSTDIR=${HOSTDIR%/}

cd ./ocdi
LOGSDIR="../logs"
if [ ! -d "$LOGSDIR" ]; then
	mkdir -m 777 -p $LOGSDIR
fi

# prepare folder
echo "- create folders"
mkdir -m 777 -p $HOSTDIR/{extrans/{error,original,spss/{file,source},toDispatch/{burncase3d,manually,unmatched}},ocdi,mysql/ocdi/data}

chmod -R 777 $HOSTDIR/extrans/to*

echo "- copy property-file(s)"
cp -f *.properties $HOSTDIR/ocdi/
#sed -i 's/^#dispatcher.from /dispatcher.from /g' $HOSTDIR/ocdi/*.properties


# init mrdb, if not exists
state=$(docker inspect -f {{.State.Running}} mrdb 2> /dev/null)
if [[ $state == "" ]]; then
	echo "- create mrdb-data-container"
	docker run --name=mrdb-data -v $HOSTDIR/mysql/ocdi/data/:/var/lib/mysql mysql/mysql-server:5.6 true \
		>> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log

	echo "- start mrdb-container"
	docker run \
		--detach \
		--name=mrdb \
		--publish=3307:3306 \
		--volumes-from=mrdb-data \
		--env=MYSQL_ROOT_PASSWORD="root" \
		mysql/mysql-server:5.6 >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log
	# waiting for database
	for i in {1..61}; do
		sleep 1 && [ "$(docker exec mrdb mysql -uroot -proot -e"SELECT 1;" 2> /dev/null | uniq)" == "1" ] && break
		[ $i -gt 60 ] && echo "database timeout, waited 60 seconds to run database." >> $LOGSDIR/install_error.log && break
	done

	echo "- add user-rights and configure timeouts for mrdb"
	docker exec mrdb mysql -uroot -proot -e"CREATE DATABASE IF NOT EXISTS g2d2 COLLATE utf8_general_ci; GRANT USAGE ON g2d2.* TO g2d2@'%' IDENTIFIED BY 'g2d2'; GRANT ALL PRIVILEGES ON g2d2.* TO g2d2@'%'; CREATE DATABASE IF NOT EXISTS fodb COLLATE utf8_general_ci; GRANT USAGE ON fodb.* TO fodb@'%' IDENTIFIED BY 'fodb'; GRANT ALL PRIVILEGES ON fodb.* TO fodb@'%'; FLUSH PRIVILEGES;" \
		 >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_out.log
	docker exec mrdb bash -c "echo \"connect_timeout=0\" >> /etc/my.cnf"
	docker exec mrdb bash -c "echo \"interactive_timeout=2000000\" >> /etc/my.cnf"
	docker exec mrdb bash -c "echo \"wait_timeout=2000000\" >> /etc/my.cnf"

	echo "- restart mrdb-container"
	docker restart mrdb >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log
	# waiting for database
	for i in {1..61}; do
		sleep 1 && [ "$(docker exec mrdb mysql -uroot -proot -e"SELECT 1;" 2> /dev/null | uniq)" == "1" ] && break
		[ $i -gt 60 ] && echo "database timeout, waited 60 seconds to run database." >> $LOGSDIR/install_error.log && break
	done
else
	echo "- mrdb-container already exists"
fi


# stop and remove ocdi, if exists
state=$(docker inspect -f {{.State.Running}} ocdi ocdi-data 2> /dev/null)
if [[ $state != "" ]]; then
	echo "- stop and remove existing ocdi-container"
	docker stop ocdi >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log
	docker rm ocdi ocdi-data >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log

	echo "- remove ocdi-image"
	docker rmi ocdi >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log
	sleep 1
fi

echo "- build new ocdi-image"
docker build -t ocdi . >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log
sleep 1

echo "- create ocdi-data-container"
docker run --name=ocdi-data -v /ocdi/extrans ocdi true
sleep 1

echo "- create and start ocdi-container"
docker run \
	--detach \
	--name=ocdi \
	--link=mrdb:mrdb \
	--link=ttp:ttp \
	--memory=2g \
	--memory-reservation=1g \
	--volume=$HOSTDIR/extrans:/ocdi/extrans \
	--volume=$HOSTDIR/ocdi/:/ocdi/config \
	--volumes-from=ocdi-data \
	--volumes-from=oc-data \
	--volume=$HOSTDIR/extrans/original:/tomcat/openclinica.data/datasets_backup \
	ocdi >> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_out.log
sleep 10

echo "- init ocdi-data-dictionary"
docker exec ocdi java -Xms64m -Xmx512m -XX:MetaspaceSize=96M -XX:MaxMetaspaceSize=256m -jar lib-1.0.9/pl-convert-tool-1.0.4.jar config/routes.properties \
	-i g2d2xml g2d2_datadictionary_common_enhanced.xml -o g2d2db \
	-i excel 2016_12_07_excel_datadictionary_burncase3d_v1.5.6.xlsx -o g2d2db \
	>> $LOGSDIR/install_out.log 2>> $LOGSDIR/install_error.log

echo "- done ocdi"
