#!/bin/bash
# ###license-information-start###
# The MOSAIC Project - Toolbox for research
# __
# Copyright (C) 2009 - 2017 The MOSAIC Project - Institute for Community Medicine 
# University Medicine of Greifswald - mosaic@uni-greifswald.de
# __
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ###license-information-end###
# author: arne.blumentritt@uni-greifswald.de

echo ""
echo "==================="
echo "=       TTP       ="
echo "==================="

HOSTDIR=$1
if [ -z "$HOSTDIR" ]; then
	read -e -p "- Set storage-path on host for config- and database-files: " -i "/opt" HOSTDIR
	if [ ! -d "$HOSTDIR" ]; then
		echo "Given hostpath does not exists '$HOSTDIR'"
		exit 1;
	fi
fi
HOSTDIR=${HOSTDIR%/}

echo "- Install MySQL"
mkdir -p $HOSTDIR/mysql/ths/data
chmod -R 777 $HOSTDIR/mysql

docker pull mysql/mysql-server:5.6 >> logs/install_out.log 2>> logs/install_error.log
docker run --name ttpdb-data -v $HOSTDIR/mysql/ths/data/:/var/lib/mysql mysql/mysql-server:5.6 true >> logs/install_out.log 2>> logs/install_error.log

echo "- Start MySQL"
docker run --name ttpdb -d --volumes-from ttpdb-data -p 3306:3306 -e MYSQL_ROOT_PASSWORD=root  mysql/mysql-server:5.6 >> logs/install_out.log 2>> logs/install_error.log
cp ./ttpdb/*.sql $HOSTDIR/mysql/ths/data/
# waiting for database
for i in {1..61}; do
	sleep 1 && [ "$(docker exec ttpdb mysql -uroot -proot -e'SELECT 1;' 2> /dev/null | uniq)" == "1" ] && break
	[ $i -gt 60 ] && echo "database timeout, waited 60 seconds to run database." >> logs/install_error.log && break
done

echo "- Execute SQL"
docker exec ttpdb /bin/sh -c "mysql -u root -proot < /var/lib/mysql/all_nvr_databases.sql" &>> logs/install_out.log
docker exec ttpdb mysql -uroot -proot -e"FLUSH PRIVILEGES;" &>> logs/install_out.log

echo "- Install JBoss"
docker build -t penndorfp/wildfly ./wildfly-image/ >> logs/install_out.log 2>> logs/install_error.log
mkdir -m 777 -p $HOSTDIR/{jboss-deploy,jboss-conf}
sleep 1
cp ./ttp/standalone.xml $HOSTDIR/jboss-conf/standalone.xml
cp ./ttp/*.ear $HOSTDIR/jboss-deploy/
cp ./ttp/*.war $HOSTDIR/jboss-deploy/
chmod 777 $HOSTDIR/jboss-deploy/*.* $HOSTDIR/jboss-conf/*.*

echo "- Start JBoss"
docker run \
	--detach \
	--name=ttp \
	--publish=8090:8080 \
	--volume=$HOSTDIR/jboss-deploy/:/opt/jboss/wildfly/standalone/deployments/ \
	--volume=$HOSTDIR/jboss-conf/standalone.xml:/opt/jboss/wildfly/standalone/configuration/standalone.xml \
	--link=ttpdb:mysql-db \
	--link=oc://openclinica.de \
	penndorfp/wildfly >> logs/install_out.log 2>> logs/install_error.log
sleep 20