#!/bin/bash
# ###license-information-start###
# The MOSAIC Project - Toolbox for research
# __
# Copyright (C) 2009 - 2017 The MOSAIC Project - Institute for Community Medicine 
# University Medicine of Greifswald - mosaic@uni-greifswald.de
# __
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ###license-information-end###
# author: arne.blumentritt@uni-greifswald.de

echo ""
echo "==================="
echo "=  Set Passwords  ="
echo "==================="
echo ""
echo "Starting Containers"
echo "-------------------"
echo "- TTPDB"
docker start ttpdb >/dev/null
echo "- OCDB"
docker start ocdb >/dev/null
echo "- OC"
docker start oc >/dev/null
echo "- OCDI"
docker start ocdi >/dev/null
echo "- MRDB"
docker start mrdb >/dev/null

read -p "- Get your storage-path on host for config- and database-files: " -i "/opt" HOSTDIR
if [ ! -d "$HOSTDIR" ]; then
	echo "Given hostpath does not exists '$HOSTDIR'"
	exit 1;
fi
HOSTDIR=${HOSTDIR%/}

echo ""
echo "TTP Dispatcher"
echo "--------------"

read -s -p "+ Current MySQL Password for User Root (default: root): " ttpRoot_old
echo ""
until docker exec ttpdb /bin/sh -c "mysql -u root -p$ttpRoot_old -e \";\"" ; do
       read -p "Wrong Password, please retry: " ttpRoot_old
done
read -p "+ New Password for User Root: " ttpRoot
docker exec ttpdb /bin/sh -c "mysql -u root -p\"$ttpRoot_old\" -e \"SET PASSWORD FOR root = PASSWORD('\"$ttpRoot\"');\""
read -p "+ New Password for User gpas: " ttpGpas
docker exec ttpdb /bin/sh -c "mysql -u root -p\"$ttpRoot\" -e \"SET PASSWORD FOR gpas = PASSWORD('\"$ttpGpas\"');\""
read -p "+ New Password for User gras: " ttpGras
docker exec ttpdb /bin/sh -c "mysql -u root -p\"$ttpRoot\" -e \"SET PASSWORD FOR gras = PASSWORD('\"$ttpGras\"');\""
read -p "+ New Password for User ttp-dispatcher: " ttpDispatcher
docker exec ttpdb /bin/sh -c "mysql -u root -p\"$ttpRoot\" -e \"SET PASSWORD FOR 'ttp-dispatcher' = PASSWORD('\"$ttpDispatcher\"');\""
read -p "+ New Password for trustee@ttp: " ttpTrustee
docker exec ttpdb /bin/sh -c "mysql -u root -p\"$ttpRoot\" -e \"UPDATE gras.user SET password = SHA2('\"$ttpTrustee\"', 256) WHERE name = 'trustee';\""

echo "- Saving new passwords in Configuration"
sed -i -r "/<security>/{ N; /<user-name>gpas/{ N; s/(<password>)(.*)(<\/password>)/\1"$ttpGpas"\3/ }}" $HOSTDIR/jboss-conf/standalone.xml
sed -i -r "/<security>/{ N; /<user-name>gras/{ N; s/(<password>)(.*)(<\/password>)/\1"$ttpGras"\3/ }}" $HOSTDIR/jboss-conf/standalone.xml
sed -i -r "/<security>/{ N; /<user-name>ttp-dispatcher/{ N; s/(<password>)(.*)(<\/password>)/\1"$ttpDispatcher"\3/ }}" $HOSTDIR/jboss-conf/standalone.xml


echo ""
echo "Open Clinica"
echo "------------"
read -p "+ New Password for User postgres: " ocPostgres
docker exec ocdb su postgres -c $"psql -c \"ALTER USER postgres WITH PASSWORD '"$ocPostgres"';\""
read -p "+ New Password for User clinica: " ocClinica
docker exec ocdb su postgres -c $"psql -c \"ALTER USER clinica WITH PASSWORD '"$ocClinica"';\""

echo "- Saving new passwords in Configuration"
docker exec oc /bin/sh -c "sed -i -r 's/(dbPass=)(.*)/\1"$ocClinica"/' /tomcat/openclinica.config/datainfo.properties"
docker exec oc /bin/sh -c "sed -i -r 's/(dbPass=)(.*)/\1"$ocClinica"/' /tomcat/openclinica-ws.config/datainfo.properties"

read -p "+ New Password for SOAP Account nvr_dispatcher: " ocNvr
ocNvr=$( echo -n $ocNvr | sha1sum | awk '{print $1}')
docker exec ocdb su postgres -c $"psql -d openclinica -c \"UPDATE public.user_account SET passwd = '"$ocNvr"' WHERE user_name = 'nvr_dispatcher';\""
docker exec ttpdb /bin/sh -c "mysql -u root -p\"$ttpRoot\" -e \"UPDATE ttp_dispatcher.configuration SET value = REPLACE(value, SUBSTRING(value, LOCATE('oc.password = ', value) + LENGTH('oc.password = '), LOCATE('\n', value, LOCATE('oc.password = ', value)) - LOCATE('oc.password = ', value) - LENGTH('oc.password = ')),'\"$ocNvr\"') WHERE configKey = 'nvr.properties';\""


echo ""
echo "OCDI"
echo "----"
read -s -p "+ Current MySQL Password for User Root (default: root): " ocdiRoot_old
echo ""
until docker exec mrdb /bin/sh -c "mysql -u root -p$ocdiRoot_old -e \";\"" ; do
       read -p "Wrong Password, please retry: " ocdiRoot_old
done
read -p "+ New Password for User Root: " ocdiRoot
docker exec mrdb /bin/sh -c "mysql -u root -p\"$ocdiRoot_old\" -e \"SET PASSWORD FOR root = PASSWORD('\"$ocdiRoot\"');\""
read -p "+ New Password for User g2d2: " ocdig2d2
docker exec mrdb /bin/sh -c "mysql -u root -p\"$ocdiRoot\" -e \"SET PASSWORD FOR g2d2 = PASSWORD('\"$ocdig2d2\"');\""
read -p "+ New Password for User fodb: " ocdifodb
docker exec mrdb /bin/sh -c "mysql -u root -p\"$ocdiRoot\" -e \"SET PASSWORD FOR fodb = PASSWORD('\"$ocdifodb\"');\""

echo "- Saving new passwords in Configuration"
sed -i -r "s/(user=g2d2&password=)([^&\n]*)/\1"$ocdig2d2"/" $HOSTDIR/ocdi/routes.properties
sed -i -r "s/(user=fodb&password=)([^&\n]*)/\1"$ocdifodb"/" $HOSTDIR/ocdi/routes.properties


echo ""
echo "Restarting Containers"
echo "---------------------"
docker stop ttp ocdi oc portal >/dev/null
echo "- OC"
docker start oc >/dev/null
sleep 15
echo "- TTP"
docker start ttp >/dev/null
sleep 45
echo "- OCDI"
docker start ocdi >/dev/null
sleep 5
echo "- Portal"
docker start portal >/dev/null
sleep 1


echo ""
echo "Complete"
echo "--------"
