package org.icmvc.nvr.service;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.ejb.Startup;

import org.icmvc.ttp.dispatcher.common.services.IConfigManager;
import org.icmvc.ttp.nvr.dispatcher.common.interfaces.IPropertiesDbResolveService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Singleton
@Startup
public class PropertiesDbResolveService implements IPropertiesDbResolveService
{
	@EJB
	private IConfigManager config;

	Map<String, Properties> properties = new HashMap<String, Properties>();
	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@PostConstruct
	public void init()
	{

	}

	private void loadProperties(String key, String propString, Charset cs)
	{
		try
		{
			InputStream stream = new ByteArrayInputStream(propString.getBytes(cs));
			Properties p = new Properties();
			p.load(stream);
			properties.put(key, p);
		} catch (IOException e)
		{
			logger.error(e.getMessage(), e);
		}
	}

	/* (non-Javadoc)
	 * @see org.icmvc.nvr.service.IPropertiesDbResolveService#getProperties(java.lang.String)
	 */
	@Override
	public Properties getProperties(String key)
	{
		return properties.get(key);
	}

	/* (non-Javadoc)
	 * @see org.icmvc.nvr.service.IPropertiesDbResolveService#getProperties(java.lang.String, java.lang.String)
	 */
	@Override
	public String getProperty(String key, String propertyKey)
	{
		if (!properties.containsKey(key))
		{
			loadProperties(key, config.get(key), StandardCharsets.UTF_8);
		}

		return properties.get(key).getProperty(propertyKey);
	}
}
