package org.icmvc.nvr.dispatcher.rest;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.util.List;

import javax.ejb.ConcurrencyManagement;
import javax.ejb.ConcurrencyManagementType;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONObject;
import org.icmvc.nvr.dispatcher.web.utils.IdConverter;
import org.icmvc.ttp.dispatcher.common.config.psn.PsnDomainTemplate;
import org.icmvc.ttp.dispatcher.common.exception.DispatcherException;
import org.icmvc.ttp.dispatcher.common.exception.ErrorCode;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto.DtoType;
import org.icmvc.ttp.dispatcher.common.model.dto.DomainValue;
import org.icmvc.ttp.dispatcher.common.model.dto.PsnAction;
import org.icmvc.ttp.dispatcher.common.model.dto.SequencePsnDTO;
import org.icmvc.ttp.dispatcher.common.model.dto.SourceValue;
import org.icmvc.ttp.dispatcher.common.model.dto.TargetValue;
import org.icmvc.ttp.dispatcher.common.services.IConfigManager;
import org.icmvc.ttp.dispatcher.common.services.IPSNConnector;
import org.icmvc.ttp.dispatcher.common.services.ISecurityManager;
import org.icmvc.ttp.dispatcher.common.workflow.IWorkflowAdapter;
import org.icmvc.ttp.dispatcher.common.workflow.ServiceWorkflows.PsnWorkflows;
import org.icmvc.ttp.nvr.dispatcher.common.Workflows;
import org.icmvc.ttp.nvr.dispatcher.common.interfaces.IPropertiesDbResolveService;
import org.icmvc.ttp.nvr.dispatcher.common.model.SiteModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Path("/nvr")
@Stateless
@ConcurrencyManagement(ConcurrencyManagementType.BEAN)
//@TransactionManagement(TransactionManagementType.BEAN)
public class NvrResource
{

	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	@EJB
	private IPSNConnector psnConnector;
	@EJB
	private IWorkflowAdapter wfm;
	@EJB
	private ISecurityManager secMan;
	@EJB
	private IConfigManager config;
	@EJB
	protected IPropertiesDbResolveService propResolver;
	@EJB
	private NvrResource testResource;
//	@Resource
//	private UserTransaction userTransaction;

	@Path("/getPsnForID")
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	public Response checkOrCreatePSN(@Context HttpServletRequest req, String json) throws DispatcherException
	{
		String apiKey = req.getHeader("apiKey");
		if (!secMan.hasPermission(apiKey, "getPsnForID", "call"))
		{
			logger.warn("apiKey '{}' has no permission to call getPsnForID", apiKey);
			throw new WebApplicationException(Response.status(Status.FORBIDDEN).build());
		}
		try
		{
			DispatcherDto dDTO = new DispatcherDto();
			JSONObject root = new JSONObject(json);
			JSONArray patients = root.getJSONArray("patients");
			for (int i = 0; i < patients.length(); i++)
			{
				JSONObject pat = patients.getJSONObject(i);
				String site = pat.getString("site");
				PsnAction action = PsnAction.GET_OR_GENERATE;
				SequencePsnDTO seqPSN = new SequencePsnDTO();
				seqPSN.setStartValue(IdConverter.concatValues(IdConverter.SEPERATOR, pat.getString("patId"), pat.getString("caseId")));
				dDTO.addDTO(DtoType.SEQUENCE_PSN, seqPSN);
				PsnDomainTemplate siteTemplate = null;
				for (PsnDomainTemplate t : config.getDispatcherConfig().getPsnConfig().getDomainTemplates())
				{
					if ("siteDomain".equalsIgnoreCase(t.getDomainName(false)))
					{
						siteTemplate = new PsnDomainTemplate();
						siteTemplate.setAlphabet(t.getAlphabet());
						siteTemplate.setCheckDigitClass(t.getCheckDigitClass());
						siteTemplate.setGenerateAlias(t.isGenerateAlias());
						siteTemplate.setDomainName(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.site.prefix") + site);
						siteTemplate.setProberties(t.getProberties() + ";PSN_PREFIX=" + site + "-");
//						userTransaction.begin();
						testResource.createDomainWithNewTx(siteTemplate);
//						userTransaction.commit();
						break;
					}
				}

				try
				{
					if (siteTemplate == null)
					{
						seqPSN.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.site.prefix") + site, 0, SourceValue.ORIGINAL_VALUE, action, DomainValue.PREDEFINED);
					} else
					{
						seqPSN.addDomain(siteTemplate.getDomainName(true), 0, SourceValue.ORIGINAL_VALUE, action, DomainValue.PREDEFINED);
					}
					seqPSN.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.bc"), 1, SourceValue.PREVIOUS_PSN, action, DomainValue.PREDEFINED);
					seqPSN.setTargetValue(TargetValue.LAST_PSN);
				} catch (DispatcherException e)
				{
					logger.error(e.getMessage(), e);
				}
//				userTransaction.begin();
				dDTO = wfm.invokeWorkflow(dDTO, PsnWorkflows.GET_ID_FROM_ID);
//				dDTO = callWorkflowWithNewTx(dDTO, PsnWorkflows.GET_ID_FROM_ID);
//				userTransaction.commit();
				String psn;

				if (seqPSN.isCompleted())
				{
					psn = seqPSN.getReturnValue();
				} else
				{
					throw new Exception("workflow did not complete");
				}

				if (!dDTO.getErrorContainer().hasErrors())
				{
					psn = seqPSN.getReturnValue();
				} else
				{
					for (ErrorCode e : dDTO.getErrorContainer().getErrors().keySet())
					{
						logger.error("ErrorCode: {}, Exception: {}, ExceptionMessage: {}", e, dDTO.getErrorContainer().getErrors().get(e).getClass().getCanonicalName(), dDTO.getErrorContainer().getErrors().get(e).getMessage());
					}

					throw new WebApplicationException(Response.status(Status.INTERNAL_SERVER_ERROR).entity("Internal Error occurred.").build());
				}

				pat.put("psn", psn);
			}
			return Response.status(Status.OK).entity(root.toString()).build();
		} catch (Exception e)
		{
			logger.error(e.getMessage(), e);
			throw new WebApplicationException(Response.status(Status.INTERNAL_SERVER_ERROR).entity("Internal Error occurred. Message: " + e.getMessage()).build());
		}
	}

	@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
	public void createDomainWithNewTx(PsnDomainTemplate siteTemplate) throws DispatcherException
	{
		psnConnector.createDomain(siteTemplate);
	}
//
//	@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
//	private DispatcherDto callWorkflowWithNewTx(DispatcherDto dto, String wf) throws DispatcherException, TimeoutException
//	{
//		return wfm.invokeWorkflow(dto, wf);
//	}

	@Path("/getAllSites")
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response getAllSites(@Context HttpServletRequest req) throws Exception
	{
		try
		{
			String apiKey = req.getHeader("apiKey");
			if (!secMan.hasPermission(apiKey, "getAllSites", "call"))
			{
				logger.warn("apiKey '{}' has no permission to call getAllSites", apiKey);
				throw new WebApplicationException(Response.status(Status.FORBIDDEN).build());
			}
//			userTransaction.begin();
			Object l = wfm.invokeWorkflow(new Object(), Workflows.GET_SITES);
//			userTransaction.commit();
			List<SiteModel> result = (List<SiteModel>) l;
			JSONArray sites = new JSONArray();
			for (SiteModel s : result)
			{
				JSONObject site = new JSONObject();
				site.put("id", s.getId());
				site.put("name", s.getName());
				site.put("oid", s.getOid());
				sites.put(site);
			}

			return Response.ok().entity(sites.toString()).build();
		} catch (DispatcherException e)
		{
			logger.error(e.getMessage(), e);
			throw new WebApplicationException(Response.status(Status.INTERNAL_SERVER_ERROR).build());
		}
	}

	@Path("/getPsnsForSite/{site}")
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public Response getPSNsForSite(@Context HttpServletRequest req, @PathParam("site") String site) throws Exception
	{
		try
		{
			String apiKey = req.getHeader("apiKey");
			if (!secMan.hasPermission(apiKey, "getPsnsForSite", "call"))
			{
				logger.warn("apiKey '{}' has no permission to call getPsnsForSite", apiKey);
				throw new WebApplicationException(Response.status(Status.FORBIDDEN).build());
			}

			JSONArray results = new JSONArray();
			// TODO
//			userTransaction.begin();
			final List<String> psns = psnConnector.getPseudonymsForDomain("nvr." + site);
//			userTransaction.commit();
			for (String string : psns)
			{
				String resultPSN = psnConnector.getPseudonymFor(string, propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.fp"));
				results.put(resultPSN);
			}

			return Response.ok().entity(results.toString()).build();
		} catch (DispatcherException e)
		{
			logger.error(e.getMessage(), e);
			throw new WebApplicationException(Response.status(Status.INTERNAL_SERVER_ERROR).build());
		}
	}
}
