package org.icmvc.nvr.dispatcher.web.controller;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.ejb.EJB;
import javax.el.ELContext;
import javax.el.MethodExpression;
import javax.el.ValueExpression;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.icmvc.commons.audit.interfaces.IAudit;
import org.icmvc.nvr.dispatcher.web.utils.MessagesBundle;
import org.icmvc.ttp.dispatcher.common.exception.DispatcherException;
import org.icmvc.ttp.dispatcher.common.exception.ErrorCode;
import org.icmvc.ttp.dispatcher.common.exception.TimeoutException;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto.DtoType;
import org.icmvc.ttp.dispatcher.common.model.dto.DomainValue;
import org.icmvc.ttp.dispatcher.common.model.dto.PsnAction;
import org.icmvc.ttp.dispatcher.common.model.dto.SequencePsnDTO;
import org.icmvc.ttp.dispatcher.common.model.dto.SourceValue;
import org.icmvc.ttp.dispatcher.common.model.dto.TargetValue;
import org.icmvc.ttp.dispatcher.common.services.IConfigManager;
import org.icmvc.ttp.dispatcher.common.workflow.IWorkflowAdapter;
import org.icmvc.ttp.nvr.dispatcher.common.Constants.ConfigProperty;
import org.icmvc.ttp.nvr.dispatcher.common.Workflows;
import org.icmvc.ttp.nvr.dispatcher.common.interfaces.IPropertiesDbResolveService;
import org.icmvc.ttp.nvr.dispatcher.common.model.SiteModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import au.com.xprime.webapp.util.InetAddressUtil;

/**
 *
 * @author Peter Penndorf
 *
 */
// @ManagedBean(name = "commonController")
public abstract class CommonController
{
	@EJB
	protected IAudit audit;
	@EJB
	protected IConfigManager config;

	@EJB
	protected IWorkflowAdapter wfm;

	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	@EJB
	protected IPropertiesDbResolveService propResolver;

	public Map<String, Object> getSessionMap()
	{
		return FacesContext.getCurrentInstance().getExternalContext().getSessionMap();
	}

	public MethodExpression createActionMethodExpression(String name)
	{
		FacesContext facesCtx = FacesContext.getCurrentInstance();
		ELContext elContext = facesCtx.getELContext();
		return facesCtx.getApplication().getExpressionFactory().createMethodExpression(elContext, name, String.class, new Class[] {});
	}

	public ValueExpression createValueExpression(String valueExpression, Class<?> valueType)
	{
		FacesContext context = FacesContext.getCurrentInstance();
		return context.getApplication().getExpressionFactory().createValueExpression(context.getELContext(), valueExpression, valueType);
	}

	public String getBaseUrl(HttpServletRequest request)
	{
		logger.debug("RequestScheme: {}", request.getScheme());
		logger.debug("RequestPort: {}", request.getServerPort());
		logger.debug("x-forwarded-proto: {}", request.getHeader("x-forwarded-proto"));
		if (StringUtils.isNotEmpty(request.getHeader("x-forwarded-proto")))
		{
			return request.getHeader("x-forwarded-proto") + "://" + request.getServerName() + request.getContextPath();
		}

		if ((request.getServerPort() == 80) || (request.getServerPort() == 443))
		{
			return request.getScheme() + "://" + request.getServerName() + request.getContextPath();
		}
		else
		{
			return request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + request.getContextPath();
		}
	}

	protected void addMessage(Severity severity, String message)
	{
		FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(severity, message, ""));
	}

	protected String getResourceText(String key)
	{
		return MessagesBundle.getBundle("msg").getString(key);
	}

	protected String getErrorText(String errorKey)
	{
		return getResourceText("app.error." + errorKey);
	}

	protected String getIp()
	{
		HttpServletRequest req = (HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		String current_ip = InetAddressUtil.getAddressFromRequest(req);

		return current_ip;
	}

	/**
	 * Invokes the workflow and catches exceptions, adds error to DTO
	 *
	 * @param dto
	 * @param workflow
	 * @return
	 */
	protected DispatcherDto invokeWorkflow(DispatcherDto dto, String workflow)
	{
		try
		{
			// logger.trace(">> invoking Workflow");
			dto = wfm.invokeWorkflow(dto, workflow);
			// logger.trace("<< Workflow done");
			return dto;
		}

		catch (DispatcherException e)
		{
			// logger.error(e.getMessage(), e);
			dto.addError(ErrorCode.DISPATCHER_ERROR, e, true);
			return dto;
		} catch (TimeoutException e)
		{
			// logger.error(e.getMessage(), e);
			dto.addError(ErrorCode.TIMEOUT, e, true);
			return dto;
		}
	}

	public List<SiteModel> getSites()
	{
		try
		{
			Object l = wfm.invokeWorkflow(new Object(), Workflows.GET_SITES);
			List<SiteModel> result = (List<SiteModel>) l;

			// for (SiteModel element : result)
			// {
			// SiteModel siteModel = element;
			// logger.debug("### Site -> oid={}, identifier={}, name={}",
			// siteModel.getOid(), siteModel.getId(), siteModel.getName());
			// }

			return result;
		} catch (Exception e)
		{
			logger.error(e.getMessage(), e);
			return new ArrayList<SiteModel>();
		}
	}

	protected DispatcherDto getPSN(String khId, String site, PsnAction action, String workflow)
	{
		DispatcherDto dDTO = new DispatcherDto();

		SequencePsnDTO seqPSN_site = new SequencePsnDTO();
		seqPSN_site.setStartValue(khId);
		dDTO.addDTO(DtoType.SEQUENCE_PSN, seqPSN_site);

		try
		{
			seqPSN_site.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, ConfigProperty.PSN_DOMAIN_PREFIX) + site, 0, SourceValue.ORIGINAL_VALUE, action, DomainValue.PREDEFINED);
			seqPSN_site.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, ConfigProperty.OC_DOMAIN), 1, SourceValue.PREVIOUS_PSN, action, DomainValue.PREDEFINED);
			seqPSN_site.setTargetValue(TargetValue.LAST_PSN);
		} catch (DispatcherException e)
		{
			logger.error(e.getMessage(), e);
			addMessage(FacesMessage.SEVERITY_WARN, getResourceText("jsf.error.internal"));
		}

		dDTO = invokeWorkflow(dDTO, workflow);

		return dDTO;
	}

}
