package org.icmvc.nvr.dispatcher.web.controller;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;
import javax.transaction.UserTransaction;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

import org.apache.commons.lang3.StringUtils;
import org.icmvc.nvr.dispatcher.web.utils.IdConverter;
import org.icmvc.ttp.dispatcher.common.config.psn.PsnDomainTemplate;
import org.icmvc.ttp.dispatcher.common.exception.DispatcherException;
import org.icmvc.ttp.dispatcher.common.exception.ErrorCode;
import org.icmvc.ttp.dispatcher.common.model.Gender;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto.DtoType;
import org.icmvc.ttp.dispatcher.common.model.dto.DomainValue;
import org.icmvc.ttp.dispatcher.common.model.dto.PsnAction;
import org.icmvc.ttp.dispatcher.common.model.dto.SequencePsnDTO;
import org.icmvc.ttp.dispatcher.common.model.dto.SourceValue;
import org.icmvc.ttp.dispatcher.common.model.dto.TargetValue;
import org.icmvc.ttp.dispatcher.common.services.IPSNConnector;
import org.icmvc.ttp.nvr.dispatcher.common.Workflows;
import org.icmvc.ttp.nvr.dispatcher.common.interfaces.IPropertiesDbResolveService;
import org.icmvc.ttp.nvr.dispatcher.common.model.SiteModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@ManagedBean(name = "createPatientController")
@ViewScoped
public class CreatePatientController extends SessionController {
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	String khID;
	Gender gender;
	Date birthdate;
	String site;
	String psn;
	String fallnr;
	String origValue;
	boolean patientExists = false;
	@EJB
	private IPSNConnector psnConnector;
	@Resource
	private UserTransaction userTransaction;
	private boolean stopSitesPoll = false;
	private List<SiteModel> siteModels;

	// @Inject
	// private Event<SessionCloseEvent> closeSessionEvent;

	@PostConstruct
	public void init() {

		Map<String, String> param = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
		khID = param.get("khId") != null ? param.get("khId") : null;
		fallnr = param.get("fallnr") != null ? param.get("fallnr") : null;
		site = param.get("site") != null ? param.get("site") : null;
		fetchSiteModels();
	}

	public boolean stopSitesPoll() {
		logger.warn("stopSitesPoll = {}",stopSitesPoll);
		return stopSitesPoll;
	}

	private void fetchSiteModels() {
		siteModels = super.getSites();
	}

	// public String logIn()
	// {
	// try
	// {
	// logInUser();
	// } catch (Exception e)
	// {
	// logger.error(e.getMessage(), e);
	// }
	//
	// return "createPatient";
	// }

	@Override
	public List<SiteModel> getSites() {
		try {
			getSession();
		} catch (Exception e) {
			logger.warn(e.getMessage());
			return new ArrayList<SiteModel>();
		}
		SiteModel site = null;
		logger.debug("loginModel is null ? {}", loginModel == null);
		logger.debug("loginModel-site is {}", loginModel != null ? loginModel.getSite() : "");
		if (siteModels == null || siteModels.isEmpty()) {
			fetchSiteModels();
		}
		List<SiteModel> localSiteList = new ArrayList<>(siteModels);
		if (loginModel != null && StringUtils.isNotEmpty(loginModel.getSite())) {

			for (SiteModel siteModel : localSiteList) {
				logger.debug("checking login-site '{}' equals list-site '{}'", loginModel.getSite(), siteModel.getId());
				if (loginModel.getSite().equals(siteModel.getId())) {
					site = siteModel;
					break;
				}
			}
		}
		if (site != null) {
			localSiteList.clear();
			localSiteList.add(site);
		}
		stopSitesPoll = true;
		return localSiteList;
	}

	public Gender[] getGenders() {
		return new Gender[] { Gender.M, Gender.F };
	}

	private boolean checkLoggedIn() {
		try {
			// logger.debug("setting cookie");
			// Faces.addResponseCookie("JSESSIONID", "haha", "/OpenClinica",
			// 3600);

			// String cookie =
			// FacesContext.getCurrentInstance().getExternalContext().getRequestHeaderMap().get("COOKIE");
			// logger.debug("COOKIE from RequestHeaderMap: {}", cookie);
			//
			// HttpServletRequest request = (HttpServletRequest)
			// FacesContext.getCurrentInstance().getExternalContext().getRequest();
			//
			// Cookie[] cookies = request.getCookies();
			//
			// for (Cookie obj : cookies)
			// {
			// logger.debug(obj.getName() + " : " + obj.getValue());
			// }

			return true;

			// if (StringUtils.isEmpty(cookie))
			// {
			// addMessage(FacesMessage.SEVERITY_WARN,
			// "You're not logged in to open clinica. Please do so and try
			// again!");
			// return false;
			// }
			// else
			// {
			// boolean loggedIn = (Boolean) wfm.invokeWorkflow(cookie,
			// Workflows.CHECK_LOGGED_IN);
			// if (!loggedIn)
			// {
			// addMessage(FacesMessage.SEVERITY_WARN,
			// "You're not logged in to open clinica. Please do so and try
			// again!");
			// return false;
			// } else
			// {
			// return true;
			// }
			// }
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new WebApplicationException(Response.status(Status.INTERNAL_SERVER_ERROR).entity("Internal Error occurred.").build());
		}
	}

	public void finish() throws Exception {
		if (checkLoggedIn()) {
			patientExists = checkOrCreatePSN(PsnAction.GET);
			checkOrCreatePSN(PsnAction.GET_OR_GENERATE);
		}
	}

	public void clear() throws IOException {
		psn = null;
		khID = null;
		gender = null;
		birthdate = null;
		site = null;
		patientExists = false;
		String baseUrl = getBaseUrl((HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest());
		FacesContext.getCurrentInstance().getExternalContext().redirect(baseUrl + FacesContext.getCurrentInstance().getViewRoot().getViewId());
	}

	public void redirect() throws IOException {
		FacesContext.getCurrentInstance().getExternalContext().invalidateSession();
		session = null;
		String url = propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.redirect.url").replace("${psn}", psn);
		logger.debug("### Redirecting to '{}'", url);
		FacesContext.getCurrentInstance().getExternalContext().redirect(url);
	}

	@TransactionAttribute(TransactionAttributeType.NEVER)
	private boolean checkOrCreatePSN(PsnAction action) throws DispatcherException, NotSupportedException, SystemException, SecurityException,
			IllegalStateException, RollbackException, HeuristicMixedException, HeuristicRollbackException {
		DispatcherDto dDTO = new DispatcherDto();
		dDTO.getExtData().put("birthdate", new SimpleDateFormat("yyyy-MM-dd").format(birthdate));
		dDTO.getExtData().put("site", site);
		dDTO.getExtData().put("gender", gender.name().toLowerCase());

		SequencePsnDTO seqPSN = new SequencePsnDTO();
		seqPSN.setStartValue(IdConverter.concatValues(IdConverter.SEPERATOR, khID, fallnr));
		dDTO.addDTO(DtoType.SEQUENCE_PSN, seqPSN);
		PsnDomainTemplate siteTemplate = null;
		for (PsnDomainTemplate t : config.getDispatcherConfig().getPsnConfig().getDomainTemplates()) {
			if ("siteDomain".equalsIgnoreCase(t.getDomainName(false))) {
				siteTemplate = new PsnDomainTemplate();
				siteTemplate.setAlphabet(t.getAlphabet());
				siteTemplate.setCheckDigitClass(t.getCheckDigitClass());
				siteTemplate.setGenerateAlias(t.isGenerateAlias());
				siteTemplate.setDomainName(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.site.prefix") + site);
				siteTemplate.setProberties(t.getProberties() + ";PSN_PREFIX=" + site + "-");
				userTransaction.begin();
				psnConnector.createDomain(siteTemplate);
				userTransaction.commit();
				break;
			}
		}

		try {
			if (siteTemplate == null) {
				seqPSN.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.site.prefix") + site, 0,
						SourceValue.ORIGINAL_VALUE, action, DomainValue.PREDEFINED);
			} else {
				seqPSN.addDomain(siteTemplate.getDomainName(true), 0, SourceValue.ORIGINAL_VALUE, action, DomainValue.PREDEFINED);
			}
			seqPSN.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.psn.domain.oc"), 1, SourceValue.PREVIOUS_PSN,
					action, DomainValue.PREDEFINED);
			seqPSN.setTargetValue(TargetValue.LAST_PSN);
		} catch (DispatcherException e) {
			logger.error(e.getMessage(), e);
		}
		userTransaction.begin();
		dDTO = invokeWorkflow(dDTO, Workflows.CREATE_PATIENT);
		userTransaction.commit();

		if (action.equals(PsnAction.GET)) {
			if (seqPSN.isCompleted()) {
				psn = seqPSN.getReturnValue();
				return true;
			} else {
				return false;
			}
		}

		if (!dDTO.getErrorContainer().hasErrors()) {
			psn = seqPSN.getReturnValue();
			return true;
		} else {
			for (ErrorCode e : dDTO.getErrorContainer().getErrors().keySet()) {
				logger.error("ErrorCode: {}, Exception: {}, ExceptionMessage: {}", e,
						dDTO.getErrorContainer().getErrors().get(e).getClass().getCanonicalName(),
						dDTO.getErrorContainer().getErrors().get(e).getMessage());
			}

			throw new WebApplicationException(Response.status(Status.INTERNAL_SERVER_ERROR).entity("Internal Error occurred.").build());
		}
	}

	public String getKhID() {
		return khID;
	}

	public void setKhID(String khID) {
		this.khID = khID;
	}

	public Gender getGender() {
		return gender;
	}

	public void setGender(Gender gender) {
		this.gender = gender;
	}

	public Date getBirthdate() {
		return birthdate;
	}

	public void setBirthdate(Date birthdate) {
		this.birthdate = birthdate;
	}

	public String getSite() {
		return site;
	}

	public void setSite(String site) {
		this.site = site;
	}

	public boolean isPatientExists() {
		return patientExists;
	}

	public void setPatientExists(boolean patientExists) {
		this.patientExists = patientExists;
	}

	public String getPsn() {
		return psn;
	}

	public void setPsn(String psn) {
		this.psn = psn;
	}

	public String getFallnr() {
		return fallnr;
	}

	public void setFallnr(String fallnr) {
		this.fallnr = fallnr;
	}

	public String getOrigValue() {
		return origValue;
	}

	public void setOrigValue(String origValue) {
		this.origValue = origValue;
	}

}
