package org.icmvc.nvr.dispatcher.web.controller;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.io.IOException;

import javax.ejb.EJB;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.ws.rs.WebApplicationException;

import org.icmvc.nvr.dispatcher.web.utils.IdConverter;
import org.icmvc.ttp.dispatcher.common.exception.DispatcherException;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto.DtoType;
import org.icmvc.ttp.dispatcher.common.model.dto.DomainValue;
import org.icmvc.ttp.dispatcher.common.model.dto.PsnAction;
import org.icmvc.ttp.dispatcher.common.model.dto.SequencePsnDTO;
import org.icmvc.ttp.dispatcher.common.model.dto.SourceValue;
import org.icmvc.ttp.dispatcher.common.model.dto.TargetValue;
import org.icmvc.ttp.dispatcher.common.workflow.ServiceWorkflows.PsnWorkflows;
import org.icmvc.ttp.nvr.dispatcher.common.Constants.ConfigProperty;
import org.icmvc.ttp.nvr.dispatcher.common.interfaces.IPropertiesDbResolveService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@ManagedBean(name = "resolvePatientController")
@ViewScoped
public class ResolvePatientController extends CommonController
{
	final Logger logger = LoggerFactory.getLogger(this.getClass());

	@EJB
	private IPropertiesDbResolveService propResolver;

	private String khID;
	private String site;
	private String psn;
	String fallnr;
	String origValue;
	private boolean completed = false;

	public void finish() throws IOException, WebApplicationException
	{

		DispatcherDto dDTO = new DispatcherDto();

		SequencePsnDTO seqPSN_site = new SequencePsnDTO();
		seqPSN_site.setStartValue(psn);
		dDTO.addDTO(DtoType.SEQUENCE_PSN, seqPSN_site);

		try
		{
			seqPSN_site.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, ConfigProperty.OC_DOMAIN), 0, SourceValue.PSN, PsnAction.RESOLVE, DomainValue.PREDEFINED);
			seqPSN_site.addDomain(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, ConfigProperty.PSN_DOMAIN_PREFIX) + site, 1, SourceValue.PREVIOUS_ORIGINAL_VALUE, PsnAction.RESOLVE, DomainValue.PREDEFINED);
			seqPSN_site.setTargetValue(TargetValue.LAST_ORIGINAL_VALUE);
		} catch (DispatcherException e)
		{
			logger.error(e.getMessage(), e);
			addMessage(FacesMessage.SEVERITY_WARN, getResourceText("jsf.error.internal"));
		}

		dDTO = invokeWorkflow(dDTO, PsnWorkflows.GET_ID_FROM_ID);

		if (!seqPSN_site.isCompleted())
		{
			addMessage(FacesMessage.SEVERITY_WARN, getResourceText("jsf.error.noKhID"));
			khID = null;
			fallnr = null;
			completed = false;
		}
		else
		{
			completed = true;
			khID = IdConverter.getPatId(seqPSN_site.getReturnValue());
			fallnr = IdConverter.getFallNr(seqPSN_site.getReturnValue());
			logger.debug("found Pat-ID '{}'", khID);
		}
	}

	public String getKhID()
	{
		return khID;
	}

	public void setKhID(String khID)
	{
		this.khID = khID;
	}

	public String getSite()
	{
		return site;
	}

	public void setSite(String site)
	{
		this.site = site;
	}

	public String getPsn()
	{
		return psn;
	}

	public void setPsn(String psn)
	{
		this.psn = psn;
	}

	public boolean isCompleted()
	{
		return completed;
	}

	public String getFallnr()
	{
		return fallnr;
	}

	public void setFallnr(String fallnr)
	{
		this.fallnr = fallnr;
	}

	public String getOrigValue()
	{
		return origValue;
	}

	public void setOrigValue(String origValue)
	{
		this.origValue = origValue;
	}

}
