package org.icmvc.nvr.dispatcher.web.controller;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.io.IOException;
import java.util.Map;

import javax.ejb.EJB;
import javax.faces.context.FacesContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.icmvc.ttp.dispatcher.common.exception.DispatcherException;
import org.icmvc.ttp.dispatcher.common.exception.TimeoutException;
import org.icmvc.ttp.dispatcher.common.exception.ValueNotFoundException;
import org.icmvc.ttp.dispatcher.common.model.DispatcherSession;
import org.icmvc.ttp.dispatcher.common.model.User;
import org.icmvc.ttp.dispatcher.common.services.ISecurityManager;
import org.icmvc.ttp.dispatcher.common.services.ISessionManager;
import org.icmvc.ttp.nvr.dispatcher.common.NvrMessageConstants;
import org.icmvc.ttp.nvr.dispatcher.common.Workflows;
import org.icmvc.ttp.nvr.dispatcher.common.interfaces.IPropertiesDbResolveService;
import org.icmvc.ttp.nvr.dispatcher.common.model.OpenClinicaLoginModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import au.com.xprime.webapp.util.InetAddressUtil;

/**
 *
 * @author Peter Penndorf
 *
 */
public class SessionController extends CommonController
{
	DispatcherSession session;
	Logger logger = LoggerFactory.getLogger(getClass());
	@EJB
	protected ISessionManager sessionManager;
	@EJB
	protected ISecurityManager secManager;
	protected OpenClinicaLoginModel loginModel;

	public void logInUser() throws DispatcherException, TimeoutException, IOException
	{
		logger.trace(">> logInUser");
		loginModel = (OpenClinicaLoginModel) wfm.invokeWorkflow(loginModel, Workflows.LOG_IN);
		logger.debug("Site of logInModel is {}", loginModel.getSite());
		if (loginModel.isLoggedIn())
		{
			HttpServletResponse r = (HttpServletResponse) FacesContext.getCurrentInstance().getExternalContext().getResponse();
			session = new DispatcherSession();
			session.setUser(new User());
			session.getUser().setUsername(loginModel.getUsername());
			session.getUser().setId(loginModel.getUsername());
			session.getData().put("IP", InetAddressUtil.getAddressFromRequest(((HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest())));
			session.setTimeOut(Integer.parseInt(propResolver.getProperty(IPropertiesDbResolveService.COMMON_NVR, "nvr.web.session.timeout"))*100);
			session.addData("loginModel", loginModel);
			sessionManager.createSession(session);
			getSessionMap().put(NvrMessageConstants.LOGINMODEL, loginModel);
			Cookie c = new Cookie("NVRSID", session.getUID());
			r.addCookie(c);
			Cookie sc = loginModel.getCookies().get(0);
			session.addData("ocSessionId", sc.getValue());
			logger.debug("setting cookie '{}' with path '{}' and value '{}'", sc.getName(), sc.getPath(), sc.getValue());
			r.addCookie(sc);
		}
		logger.trace("<< logInUser");
//		String baseUrl = getBaseUrl((HttpServletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest());
//		FacesContext.getCurrentInstance().getExternalContext().redirect(baseUrl+FacesContext.getCurrentInstance().getViewRoot().getViewId());
	}

	protected DispatcherSession getSessionBySid(String sid)
	{
		session = sessionManager.getSessionBySessionId(sid);

		return session;
	}

	public boolean isLoggedIn()
	{
		if(loginModel==null)
		{
			loginModel = new OpenClinicaLoginModel();
		}
		try
		{
			if(session==null)
			{
				getSession();
			}
			//Prüfen, ob auch die gleiche IP verwendet wurde!!!
			boolean ocLoggedIn = false;
			
			if(session!=null)
			{
				ocLoggedIn = (Boolean) wfm.invokeWorkflow(session.get("ocSessionId"), Workflows.CHECK_LOGGED_IN);
				if(!ocLoggedIn && session.isValid())
				{
					HttpServletResponse r = (HttpServletResponse) FacesContext.getCurrentInstance().getExternalContext().getResponse();
					Cookie c = getCookie("NVRSID");
					c.setMaxAge(0);
					r.addCookie(c);
					loginModel= new OpenClinicaLoginModel();
					logger.debug("Removed NVRSID Cookie");
				}
			}
			loginModel.setLoggedIn(session == null ? false : session.isValid() & ocLoggedIn);
			logger.debug("logged in ? {}", loginModel.isLoggedIn());
		}catch (Exception e)
		{
			logger.warn(e.getMessage());
			loginModel.setLoggedIn(false);
		}
		return loginModel.isLoggedIn();
	}
	
	public void removeSession()
	{
		sessionManager.deleteSession(session.getUID());
		session = null;
	}

	protected void getSession() throws ValueNotFoundException
	{
		Cookie c = getCookie("NVRSID");
		getSessionBySid(c.getValue());
		if(session!=null)
		{
			loginModel = (OpenClinicaLoginModel) session.getData().get("loginModel");
			logger.debug("Site of logInModel is {}", loginModel.getSite());
		}
	}

	protected Cookie getCookie(String id) throws ValueNotFoundException
	{
		Map<String, Object> cookies = FacesContext.getCurrentInstance().getExternalContext().getRequestCookieMap();
		if (cookies.containsKey(id))
		{
			if (cookies.get(id) instanceof Cookie)
			{
				return (Cookie) cookies.get(id);
			}
		}
		throw new ValueNotFoundException("No cookie found for id '" + id + "'");
	}

	public OpenClinicaLoginModel getLoginModel()
	{
		return loginModel;
	}

	public void setLoginModel(OpenClinicaLoginModel loginModel)
	{
		this.loginModel = loginModel;
	}

}
