package org.icmvc.nvr.dispatcher.web.validation;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */


import java.io.Serializable;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.FacesConverter;
import javax.faces.validator.ValidatorException;

import org.icmvc.nvr.dispatcher.web.utils.MessagesBundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@FacesConverter("org.icmvc.jsf.BirthdateConverter")
public class BirthdateValidateConverter implements Converter, Serializable
{
	/**
	 *
	 */
	private static final long serialVersionUID = -8872122724828626492L;
	private final Logger logger = LoggerFactory.getLogger(this.getClass());
	private Integer minAge;
	private Integer maxAge;

	private void validate(Date value) throws ValidatorException, ParseException
	{
		if (value != null)
		{
			if (value instanceof Date)
			{
				Date d = value;
				Validator v = new Validator();
				if (!v.isValidAge(d, minAge, maxAge))
				{
					String valMSG = MessagesBundle.getBundle("msg").getString("jsf.form.common.birthdate.error.age");
					valMSG = valMSG.replace("{min}", minAge.toString());
					valMSG = valMSG.replace("{max}", maxAge.toString());
					FacesMessage msg = new FacesMessage(valMSG);
					msg.setSeverity(FacesMessage.SEVERITY_ERROR);
					throw new ValidatorException(msg);
				}
			}
			else
			{
				logger.error("given value is not of type Date");
			}
		}
	}

	@Override
	public Object getAsObject(FacesContext context, UIComponent component, String value)
	{
		SimpleDateFormat sf = new SimpleDateFormat(component.getAttributes().get("pattern").toString());
		sf.setLenient(false);
		try
		{
			minAge = Integer.parseInt(component.getAttributes().get("minAge").toString());
			maxAge = Integer.parseInt(component.getAttributes().get("maxAge").toString());

			Date d = sf.parse(value);
			validate(d);

			return d;
		} catch (ValidatorException e)
		{
			throw new ConverterException(e.getFacesMessage());
		} catch (NullPointerException e)
		{
			FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.form.common.birthdate.errormsg"));
			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
			throw new ConverterException(msg);
		} catch (Exception e)
		{
			FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.form.common.birthdate.errormsg"));
			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
			throw new ConverterException(msg);
		}
	}

	@Override
	public String getAsString(FacesContext context, UIComponent component, Object value)
	{
		SimpleDateFormat sf = new SimpleDateFormat(component.getAttributes().get("pattern").toString());
		if (value instanceof Date)
		{
			return sf.format(value);
		}
		else
		{
			FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.form.common.birthdate.errormsg"));
			msg.setSeverity(FacesMessage.SEVERITY_ERROR);
			throw new ConverterException(msg);
		}
	}

}
