package org.icmvc.nvr.dispatcher.web.validation;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.FacesValidator;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import org.icmvc.nvr.dispatcher.web.utils.MessagesBundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@FacesValidator("org.icmvc.jsf.DateValidator")
public class DateValidator implements Validator
{
	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Override
	public void validate(FacesContext context, UIComponent comp, Object value) throws ValidatorException
	{
		if (value instanceof Date)
		{
			Date d = (Date) value;
			Object fAllowed = comp.getAttributes().get("futureAllowed");
			String pattern = (String) comp.getAttributes().get("pattern");
			if ("false".equalsIgnoreCase(fAllowed.toString()))
			{
				if (d.after(new Date()))
				{
					FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.common.validation.date.futureNotAllowed").replace("{field}", comp.getAttributes().get("label").toString()));
					msg.setSeverity(FacesMessage.SEVERITY_ERROR);
					throw new ValidatorException(msg);
				}
			}

			Object notBeforeYear = comp.getAttributes().get("notBeforeYear");
			if (notBeforeYear != null)
			{
				Integer years = 0;
				try
				{
					years = Integer.parseInt(notBeforeYear.toString());
				} catch (NumberFormatException e)
				{
					logger.error("The passed value for 'notBeforeYear' could not be parsed as integer");
					FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.common.validation.parsingError"));
					msg.setSeverity(FacesMessage.SEVERITY_ERROR);
					throw new ValidatorException(msg);
				}
				final Calendar date = Calendar.getInstance();
				date.setTime(d);
				int dateYears = date.get(Calendar.YEAR);

				logger.trace("{} - {} = {}", dateYears, years, Integer.toString(dateYears - years).toString());
				if (dateYears < years)
				{
					FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.common.validation.date.notBeforeYear").replace("{field}", comp.getAttributes().get("label").toString()).replace("{year}", years.toString()));
					msg.setSeverity(FacesMessage.SEVERITY_ERROR);
					throw new ValidatorException(msg);
				}
			}

			Object notBeforeDate = comp.getAttributes().get("notBeforeDate");
			if (notBeforeDate != null)
			{
				Date notBefore;
				SimpleDateFormat format = new SimpleDateFormat(pattern);
				try
				{
					notBefore = format.parse(notBeforeDate.toString());
				} catch (Exception e)
				{
					logger.error("The passed value for 'notBeforeDate' could not be parsed as date", e);
					FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.common.validation.parsingError"));
					msg.setSeverity(FacesMessage.SEVERITY_ERROR);
					throw new ValidatorException(msg);
				}
				final Calendar date = Calendar.getInstance();
				date.setTime(d);
				long inputDate = date.getTimeInMillis();

				// logger.trace("{} - {} = {}", inputDate, notBefore,
				// Integer.toString(inputDate - notBefore).toString());
				if (inputDate < notBefore.getTime())
				{
					FacesMessage msg = new FacesMessage(MessagesBundle.getBundle("msg").getString("jsf.common.validation.date.notBeforeDate").replace("{field}", comp.getAttributes().get("label").toString()).replace("{date}", format.format(notBefore)));
					msg.setSeverity(FacesMessage.SEVERITY_ERROR);
					throw new ValidatorException(msg);
				}
			}

		} else
		{
			logger.error("Validation can not be done with valueType of '{}'", value.getClass().getCanonicalName());
		}
	}

}
