package org.icmvc.nvr.dispatcher.web.validation;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;

public class Validator
{
	public boolean isValidStringLength(final String str, final int maxLength)
	{
		return str == null || str.length() <= maxLength;
	}

	public boolean isValidStringLengthMin(final String str, final int minLength)
	{
		return str == null || str.length() >= minLength;
	}

	public boolean isEmpty(final String str)
	{
		return str == null || str.trim().equals("");
	}

	public boolean isBoolean(final String str)
	{
		return "true".equals(str) || "false".equals(str) || str == null;
	}

	public boolean isValidDate(final String str, final String format)
	{
		try
		{
			if (str == null)
			{
				return true;
			}
			final SimpleDateFormat dateFormat = new SimpleDateFormat(format);
			dateFormat.setLenient(false);
			dateFormat.parse(str);
			return true;
		}
		catch (final ParseException pe)
		{
		}
		return false;
	}

	public boolean isNumber(final String str)
	{
		if (str == null)
		{
			return true;
		}
		// Wenn der String mit Minus anfängt, wird das zweite Zeichen des
		// Strings übergeben, ansonsten der String, da isNumeric keine negativen
		// Zahlen mag.
		return StringUtils.isNumeric((str.startsWith("-")) ? str.substring(1) : str);
	}

	public boolean isFloat(String str)
	{
		if (str == null)
		{
			return true;
		}
		str = str.replace(',', '.');
		try
		{
			Float.parseFloat(str);
			return true;
		}
		catch (final NumberFormatException pe)
		{
			return false;
		}
	}

	public boolean isValidRange(String value, final float min, final float max)
	{
		if (value == null)
		{
			return true;
		}
		value = value.replace(',', '.');
		float val;
		try
		{
			val = Float.parseFloat(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return min <= val && val <= max;
	}

	public boolean isValidRange(String value, final double min, final double max)
	{
		if (value == null)
		{
			return true;
		}
		value = value.replace(',', '.');
		double val;
		try
		{
			val = Double.parseDouble(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return min <= val && val <= max;
	}

	public boolean isValidRange(String value, final double min, final double max, final double ignore)
	{
		if (value == null)
		{
			return true;
		}
		value = value.replace(',', '.');
		double val;
		try
		{
			val = Double.parseDouble(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return (min <= val && val <= max) || val == ignore;
	}

	public boolean isValidRange(String value, final float min, final float max, final float ignore)
	{
		if (value == null)
		{
			return true;
		}
		value = value.replace(',', '.');
		float val;
		try
		{
			val = Float.parseFloat(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return (min <= val && val <= max) || val == ignore;
	}

	public boolean isValidRange(String value, final float min, final float max, final float ignore, final float ignore2)
	{
		if (value == null)
		{
			return true;
		}
		value = value.replace(',', '.');
		float val;
		try
		{
			val = Float.parseFloat(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return (min <= val && val <= max) || val == ignore || val == ignore2;
	}

	public boolean isValidRange(String value, final int min, final int max, final int ignore, final int ignore2)
	{
		if (value == null)
		{
			return true;
		}
		value = value.replace(',', '.');
		int val;
		try
		{
			val = Integer.parseInt(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return (min <= val && val <= max) || val == ignore || val == ignore2;
	}

	public boolean isValidRange(final String value, final String dateFormat, final Date min, final Date max)
	{
		if (value == null)
		{
			return true;
		}
		final SimpleDateFormat sDateFormat = new SimpleDateFormat(dateFormat);
		Date val = null;
		try
		{
			val = sDateFormat.parse(value);
		}
		catch (final Exception e)
		{
			return false;
		}
		return min.getTime() <= val.getTime() && val.getTime() <= max.getTime();
	}

	public boolean isValidRange(final String value, final String dateFormat, final String min, final Date max, final int ignore)
	{
		Date minimum;
		if (value == null || ("" + ignore).equals(value))
		{
			return true;
		}

		try
		{
			minimum = new SimpleDateFormat(dateFormat).parse(min);
			final SimpleDateFormat sDateFormat = new SimpleDateFormat(dateFormat);
			Date val = null;
			try
			{
				val = sDateFormat.parse(value);
			}
			catch (final Exception e)
			{
				return false;
			}
			return minimum.getTime() <= val.getTime() && val.getTime() <= max.getTime();
		}
		catch (final ParseException pe)
		{
			return false;
		}
	}

	public boolean isValidRange(final String value, final String dateFormat, final String min, final Date max)
	{
		Date minimum;
		try
		{
			minimum = new SimpleDateFormat(dateFormat).parse(min);

			if (value == null)
			{
				return true;
			}
			final SimpleDateFormat sDateFormat = new SimpleDateFormat(dateFormat);
			Date val = null;
			try
			{
				val = sDateFormat.parse(value);
			}
			catch (final Exception e)
			{
				return false;
			}
			return minimum.getTime() <= val.getTime() && val.getTime() <= max.getTime();
		}
		catch (final ParseException pe)
		{
			return false;
		}
	}

	public boolean isValidPlaces(String eingabe, final int vorkomma, final int nachkomma)
	{

		if (eingabe == null)
		{
			return true;
		}
		// Komma durch Punkt ersetzen
		eingabe = eingabe.replace(',', '.');
		final int gesamtLaenge = vorkomma + nachkomma + 1;
		final int kommaPosition = eingabe.indexOf('.');

		return (eingabe.length() == gesamtLaenge) && (kommaPosition == vorkomma) && (eingabe.length() - (kommaPosition + 1) == nachkomma);

	}

	public boolean isValidTimeOrder(final Date before, final Date after)
	{
		if (before == null || after == null)
		{
			return false;
		}
		else
		{
			return before.before(after) || before.equals(after);
		}
	}

	private int getAge(final Date dateOfBirth)
	{
		int age = 0;
		final Calendar born = Calendar.getInstance();
		final Calendar now = Calendar.getInstance();
		if (dateOfBirth != null)
		{
			now.setTime(new Date());
			born.setTime(dateOfBirth);
			if (born.after(now))
			{
				return -1;
			}
			age = now.get(Calendar.YEAR) - born.get(Calendar.YEAR);
			if (now.get(Calendar.DAY_OF_YEAR) < born.get(Calendar.DAY_OF_YEAR))
			{
				age -= 1;
			}
		}
		return age;
	}

	/**
	 * Checks if the given value matches the regexp
	 * 
	 * @param value
	 *            should be an Object of
	 *            int,long,float,double,String,decimal...every type that can be
	 *            easily converted to a String
	 * @param regex
	 * @return
	 */
	public boolean isValidRegex(final Object value, final String regex)
	{
		return value.toString().matches(regex);
	}

	/**
	 * Checks if the given value matches the regexp
	 * 
	 * @param value
	 * @param regex
	 * @return
	 */
	public boolean isValidRegex(final String value, final String regex)
	{
		return value.matches(regex);
	}

	/**
	 * Checks if the given birthdate is between the min and max age
	 * (birthdate=max|min returns true)
	 * 
	 * @param value
	 *            birthdate as string
	 * @param pattern
	 *            for the date format (most-likely dd.MM.yyyy)
	 * @param min
	 *            defines the minimum age
	 * @param max
	 *            define the maximum age
	 * @return
	 */
	public boolean isValidAge(final String value, final String pattern, final int min, final int max)
	{
		try
		{
			final SimpleDateFormat dateFormat = new SimpleDateFormat(pattern);
			dateFormat.setLenient(false);
			final Date d = dateFormat.parse(value);

			final int age = getAge(d);

			return (age >= min & age <= max);

		}
		catch (final ParseException e)
		{
			return false;
		}
	}

	/**
	 * Checks if the given birthdate is between the min and max age
	 * (birthdate=max|min returns true)
	 * 
	 * @param value
	 *            birthdate as string
	 * @param pattern
	 *            for the date format (most-likely dd.MM.yyyy)
	 * @param min
	 *            defines the minimum age
	 * @param max
	 *            define the maximum age
	 * @return
	 */
	public boolean isValidAge(final Date d, final int min, final int max)
	{
		final int age = getAge(d);

		return (age >= min & age <= max);
	}

}
