/**
 * 
 */
package org.icmvc.nvr.dispatcher.wfm.camel;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.util.Calendar;
import java.util.EventObject;
import java.util.concurrent.ConcurrentHashMap;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.camel.management.event.ExchangeCompletedEvent;
import org.apache.camel.management.event.ExchangeCreatedEvent;
import org.apache.camel.management.event.ExchangeSendingEvent;
import org.apache.camel.support.EventNotifierSupport;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto;
import org.icmvc.ttp.dispatcher.common.services.ISessionManager;
import org.icmvc.ttp.nvr.dispatcher.common.NvrMessageConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;

/**
 * @author Peter Penndorf
 * 
 */
@Named
public class CamelEventListener extends EventNotifierSupport
{
	volatile ConcurrentHashMap<String, Long> exchanges = new ConcurrentHashMap<String, Long>();
	Logger logger = LoggerFactory.getLogger(getClass());
	@Inject
	ISessionManager sManager;

	@Override
	public boolean isEnabled(EventObject event)
	{
		return true;
	}

	@Override
	public void notify(EventObject event) throws Exception
	{
		// logger.trace("Event is of type '{}'",
		// event.getClass().getCanonicalName());

		if (event instanceof ExchangeCreatedEvent)
		{
			ExchangeCreatedEvent e = (ExchangeCreatedEvent) event;
			exchanges.put(e.getExchange().getExchangeId(), Calendar.getInstance().getTimeInMillis());
		} else if (event instanceof ExchangeCompletedEvent)
		{
			ExchangeCompletedEvent e = (ExchangeCompletedEvent) event;
			String exId = e.getExchange().getExchangeId();
			if (exchanges.containsKey(exId))
			{
				Long time = Calendar.getInstance().getTimeInMillis() - exchanges.get(exId);
				// logger.info("### ExchangeMessage '" + exId + "' in Route '" +
				// e.getExchange().getFromRouteId() + "' (Endpoint: " +
				// e.getExchange().getFromEndpoint().getEndpointUri() +
				// ") took " + time + "ms to complete");
				exchanges.remove(exId);
			}
		} else if (event instanceof ExchangeSendingEvent)
		{
			// logger.trace("### SendingEvent found");
			if (((ExchangeSendingEvent) event).getExchange().getIn().getBody() instanceof DispatcherDto)
			{
				// logger.trace("### SendingEvent - DispatcherDto found");
				DispatcherDto dto = (DispatcherDto) ((ExchangeSendingEvent) event).getExchange().getIn().getBody();
				if (dto.getExtData().containsKey(NvrMessageConstants.MDC_ID))
				{
					// logger.trace("### SendingEvent - DispatcherDto - MDC_ID found");
					MDC.put("mdc", dto.getExtData().get(NvrMessageConstants.MDC_ID).toString());
				}
			}
		}
	}

	@Override
	protected void doStart() throws Exception
	{
		// filter out unwanted events
		setIgnoreCamelContextEvents(true);
		setIgnoreServiceEvents(true);
		setIgnoreRouteEvents(true);
		setIgnoreExchangeCreatedEvent(false);
		setIgnoreExchangeCompletedEvent(false);
		setIgnoreExchangeFailedEvents(true);
		setIgnoreExchangeRedeliveryEvents(true);
		setIgnoreExchangeSentEvents(false);
		setIgnoreExchangeSendingEvents(false);
	}

	@Override
	protected void doStop() throws Exception
	{
		// TODO Auto-generated method stub

	}

}
