package org.icmvc.nvr.dispatcher.wfm.camel;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.util.concurrent.Future;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.ConcurrencyManagement;
import javax.ejb.ConcurrencyManagementType;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.inject.Inject;

import org.icmvc.ttp.dispatcher.common.exception.DispatcherException;
import org.icmvc.ttp.dispatcher.common.exception.TimeoutException;
import org.icmvc.ttp.dispatcher.common.model.dto.DispatcherDto;
import org.icmvc.ttp.dispatcher.common.workflow.IWorkflowAdapter;
import org.icmvc.ttp.dispatcher.wfm.IDispatcherWorkflowManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Singleton
@Startup
@Local(IWorkflowAdapter.class)
@ConcurrencyManagement(ConcurrencyManagementType.BEAN)
public class WorkflowManager implements IWorkflowAdapter
{
	@EJB
	private volatile IDispatcherWorkflowManager wfManager;
	Logger logger = LoggerFactory.getLogger(getClass());
	@Inject
	Workflow wf;
	@Inject
	CamelEventListener listener;

	@PostConstruct
	public void init()
	{
		try
		{
			wfManager.addRoutes(wf);
			wfManager.addNotifier(listener);
			wfManager.enableTracing(false);
			wfManager.start();
		}
		catch (Exception e)
		{
			logger.error(e.getMessage(), e);
		}
	}

	@PreDestroy
	public void shutdown()
	{
		// wfManager.shutdown();
	}

	@Override
	public DispatcherDto invokeWorkflow(DispatcherDto message, String workflowID) throws DispatcherException, TimeoutException
	{
		message = wfManager.invokeWorkflow(message, workflowID);
		if (message == null)
		{
			// TODO: ExceptionHandling hier
			// message.getMessage().put(key, value)
		}

		return message;
	}

	@Override
	public Object invokeWorkflow(Object message, String workflowID) throws DispatcherException, TimeoutException
	{
		return wfManager.invokeWorkflow(message, workflowID);
	}

	@Override
	public <T> Future<T> invokeWorkflowAsync(Object message, String endpoint, Class<T> c) throws DispatcherException, TimeoutException
	{
		return wfManager.invokeWorkflowAsync(message, endpoint, c);
	}
}
