package org.icmvc.nvr.test;

/*
 * ###license-information-start###
 * Toolbox for Research - TTP Dispatcher
 * __
 * Copyright (C) 2017 The MOSAIC Project - Institut fuer Community Medicine der
 * 							Universitaetsmedizin Greifswald - mosaic@uni-greifswald.de
 * __
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ###license-information-end###
 */

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.camel.Exchange;
import org.apache.camel.LoggingLevel;
import org.apache.camel.Message;
import org.apache.camel.Processor;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.properties.PropertiesComponent;
import org.apache.camel.main.Main;
import org.apache.camel.processor.aggregate.AggregationStrategy;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.cookie.Cookie;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.icmvc.ttp.nvr.dispatcher.common.Workflows;
import org.icmvc.ttp.nvr.dispatcher.common.model.OpenClinicaLoginModel;
import org.icmvc.ttp.nvr.dispatcher.common.model.SiteModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MainApp
{
	final static Logger logger = LoggerFactory.getLogger(MainApp.class);

	/**
	 * A main() so we can easily run these routing rules in our IDE
	 */
	public static void main(final String... args) throws Exception
	{

		final Main main = new Main();
		main.enableHangupSupport();
		main.addRouteBuilder(new RouteBuilder()
		{
			@Override
			public void configure() throws Exception
			{
				// from("file://soap_in?delete=false&noop=true").convertBodyTo(String.class).to("vm:parseSites");

				// from("timer:getSitesList?repeatCount=1")
				// .setBody(constant("apsindfpi"))
				// .setHeader("birthdate", constant("2000-12-15"))
				// .setHeader("gender", constant("m"))
				// .setHeader("siteId", constant("sit2"))
				// .to(Workflows.CREATE_PATIENT);

				intercept().to("log:interception?showAll=true");

				// from("timer:test?repeatCount=1")
				// .setBody(constant("JSESSIONID=B3A43BDA27C4BB45F77D49204389C4E"))
				// .to("direct:checkLoggedIn");

				from("timer:test?repeatCount=1").to(Workflows.GET_SITES);

				from("direct:logIn")
						// .setBody(constant("j_username=root&j_password=openclinica&submit=Login"))
						// .setHeader(Exchange.HTTP_METHOD, constant("POST"))
						// .setHeader(Exchange.CONTENT_TYPE,
						// constant("application/x-www-form-urlencoded"))
						// .doTry()
						// .to("http4:{{dbprop:oc.url.logIn}}?bridgeEndpoint=true")
						// .doCatch(HttpOperationFailedException.class)
						// .choice().when(simple("${exception.statusCode} == '302'"))
						// .convertBodyTo(String.class, "UTF-8")
						// .log(simple("Body: ${body}").toString())
						// .log(simple("COOKIE: ${out.headers.Set-Cookie}").toString())
						// .to("file:http_output.txt")
						.setHeader("url", simple("http:{{dbprop:oc.url.logIn}}"))
						.setBody(constant(new OpenClinicaLoginModel("root", "openclinica")))
						.process(new Processor()
						{
							@Override
							public void process(Exchange exchange) throws Exception
							{
								Message in = exchange.getIn();
								OpenClinicaLoginModel m = (OpenClinicaLoginModel) in.getBody();
								BasicCookieStore store = new BasicCookieStore();
								CloseableHttpClient httpclient = HttpClients.custom().setDefaultCookieStore(store).build();
								HttpPost httpPost = new HttpPost(in.getHeader("url").toString());
								List<NameValuePair> nvps = new ArrayList<NameValuePair>();
								nvps.add(new BasicNameValuePair("j_username", m.getUsername()));
								nvps.add(new BasicNameValuePair("j_password", m.getPassword()));
								nvps.add(new BasicNameValuePair("submit", "Login"));
								httpPost.setEntity(new UrlEncodedFormEntity(nvps));
								CloseableHttpResponse response = httpclient.execute(httpPost);
								if (response.getStatusLine().getStatusCode() == 302 && !response.getHeaders("Location")[0].getValue().contains("error"))
								{
									for (Cookie c : store.getCookies())
									{
										javax.servlet.http.Cookie sc = new javax.servlet.http.Cookie(c.getName(), c.getValue());
										sc.setDomain(c.getDomain());
										sc.setPath(c.getPath());
										if (c.getExpiryDate() != null)
										{
											sc.setMaxAge((int) ((c.getExpiryDate().getTime() - (new Date().getTime())) / 1000));
										}
										m.getCookies().add(sc);
									}
									m.setLoggedIn(true);
									in.setBody(m);
								}
								else
								{
									m.setLoggedIn(false);
									exchange.getIn().setBody(m);
								}
								response.close();
								httpclient.close();
							}
						})
						.log(simple("LoggedIn ? ${in.body.isLoggedIn}").toString());
				// .otherwise().throwException(new Exception());

				from("direct:checkLoggedIn")
						.setHeader(Exchange.HTTP_METHOD, constant("GET"))
						.setHeader("COOKIE", simple("${body}"))
						// .setHeader("referrer",
						// constant("http://141.53.75.39/OpenClinica/pages/login/login"))
						.to("http4:{{dbprop:oc.url.testLoggedIn}}")
						.convertBodyTo(String.class, "UTF-8")
						.log(simple("COOKIE: ${in.headers.COOKIE}").toString())
						.choice().when(body().contains("loginBox"))
						.log(constant("### not logged in").toString())
						.setBody(constant(false))
						.otherwise()
						.log(constant("### logged in").toString())
						.setBody(constant(true));

				from("direct:setCommonHeader")
						.setHeader("usernameToken", simple("{{dbprop:oc.usernameToken}}"))
						.setHeader("username", simple("{{dbprop:oc.username}}"))
						.setHeader("password", simple("{{dbprop:oc.password}}"))
						.setHeader("eventId", simple("{{dbprop:oc.event.default.id}}"))
						.setHeader("enrDate", simple("date:now:yyyy-MM-dd"))
						.setHeader("startDate", simple("date:now:yyyy-MM-dd"))
						.setHeader("startTime", simple("date:now:hh:MM"))
						.setHeader("studyId", simple("{{dbprop:oc.study.default.id}}"));

				from("direct:checkServiceResponse").convertBodyTo(String.class, "UTF-8")
						.log(LoggingLevel.DEBUG, logger, "checking response '${body}'")
						.setHeader("responseFaultString", xpath("//*[local-name()='faultstring']/text()"))
						.setHeader("responseResult", xpath("//*[local-name()='result']/text()"))
						.choice().when(header("responseFaultString").isEqualTo(null))
						.log(LoggingLevel.ERROR, logger, "Service returned error meassage '${header.responseFaultString}'")
						.throwException(new Exception())
						.otherwise().when(header("responseResult").isNotEqualTo("Success"))
						.setHeader("responseMessage", xpath("//*[local-name()='error']/text()"))
						.log(LoggingLevel.ERROR, logger, "Call of Service was not successfully. Message: ${header.responseMessage}")
						.throwException(new Exception());

				from("direct:isStudySubject")
						.to("direct:setCommonHeader")
						.setHeader("psn", simple("${body}"))
						.setHeader(Exchange.HTTP_METHOD, simple("POST"))
						.setHeader(Exchange.CONTENT_TYPE, constant("text/xml"))
						.to("http4:{{dbprop:oc.url.studySubjectService}}").convertBodyTo(String.class, "UTF-8")
						.setHeader("responseResult", xpath("//*[local-name()='result']/text()"))
						.choice().when(header("responseResult").isEqualTo("Fail"))
						.log("Subject not found");

				from(Workflows.CREATE_PATIENT)
						.to("direct:setCommonHeader")
						.setHeader("psn", simple("${body}"))
						.to("freemarker:soap_templates/createPatient.xml").convertBodyTo(String.class, "UTF-8")
						.setHeader(Exchange.HTTP_METHOD, simple("POST"))
						.setHeader(Exchange.CONTENT_TYPE, constant("text/xml"))
						.to("http4:{{dbprop:oc.url.studySubjectService}}")
						.to("direct:checkServiceResponse")
						.to("freemarker:soap_templates/registerEvent.xml").convertBodyTo(String.class, "UTF-8")
						.setHeader(Exchange.HTTP_METHOD, simple("POST"))
						.setHeader(Exchange.CONTENT_TYPE, constant("text/xml"))
						.to("http4:{{dbprop:oc.url.registerEvent}}")
						.to("direct:checkServiceResponse");

				from(Workflows.GET_SITES)
						.to("direct:setCommonHeader")
						.to("freemarker:soap_templates/getSitesList.xml").convertBodyTo(String.class, "UTF-8")
						.setHeader(Exchange.HTTP_METHOD, simple("POST"))
						.setHeader(Exchange.CONTENT_TYPE, constant("text/xml"))
						.to("http4:{{dbprop:oc.url.getSites}}")
						.to("direct:checkServiceResponse")
						.to("direct:parseSites");

				// from("direct:parseSites")
				// .setBody(xpath("//*[local-name()='study'][*[local-name()='identifier']='default-study']"))
				// .split(body().tokenizeXML("site", ""), new
				// AggregationStrategy()
				// {
				// @Override
				// public Exchange aggregate(Exchange oldExchange, Exchange
				// newExchange)
				// {
				// if (oldExchange == null)
				// {
				// Message m = newExchange.getIn();
				// SiteModel s = new SiteModel(m.getHeader("id").toString(),
				// m.getHeader("name").toString(),
				// m.getHeader("oid").toString());
				// newExchange.getIn().setBody(new ArrayList<SiteModel>());
				// ((ArrayList<SiteModel>)
				// newExchange.getIn().getBody()).add(s);
				// return newExchange;
				// }
				//
				// Message m = newExchange.getIn();
				// SiteModel s = new SiteModel(m.getHeader("id").toString(),
				// m.getHeader("name").toString(),
				// m.getHeader("oid").toString());
				// ((ArrayList<SiteModel>)
				// oldExchange.getIn().getBody()).add(s);
				// return oldExchange;
				// }
				// }).log(simple("${body}").toString())
				// .setHeader("id", xpath("//identifier/text()").stringResult())
				// .setHeader("oid", xpath("//oid/text()").stringResult())
				// .setHeader("name", xpath("//name/text()").stringResult())
				// .end().process(new Processor()
				// {
				// @Override
				// public void process(Exchange exchange) throws Exception
				// {
				// Message m = exchange.getIn();
				// Object b = m.getBody();
				// if (b != null && b instanceof List)
				// {
				// ArrayList<SiteModel> sites = (ArrayList<SiteModel>) b;
				// for (Object element : sites)
				// {
				// SiteModel siteModel = (SiteModel) element;
				// logger.debug("### Site -> oid={}, identifier={}, name={}",
				// siteModel.getOid(), siteModel.getId(), siteModel.getName());
				// }
				// } else
				// {
				// logger.error("No SiteModel");
				// }
				// }
				// });
				// }

				from("direct:parseSites")
						.setBody(xpath("//*[local-name()='study'][*[local-name()='identifier']='Verbrennungsregister']"))
						.split(body().tokenizeXML("site", ""), new AggregationStrategy()
						{
							@Override
							public Exchange aggregate(Exchange oldExchange, Exchange newExchange)
							{
								if (oldExchange == null)
								{
									Message m = newExchange.getIn();
									SiteModel s = new SiteModel(m.getHeader("id").toString(), m.getHeader("name").toString(), m.getHeader("oid").toString());
									logger.debug(s.toString());
									newExchange.getIn().setBody(new ArrayList<SiteModel>());
									((ArrayList<SiteModel>) newExchange.getIn().getBody()).add(s);
									return newExchange;
								}

								Message m = newExchange.getIn();
								SiteModel s = new SiteModel(m.getHeader("id").toString(), m.getHeader("name").toString(), m.getHeader("oid").toString());
								((ArrayList<SiteModel>) oldExchange.getIn().getBody()).add(s);

								logger.debug(s.toString());

								return oldExchange;
							}
						})
						// .log(LoggingLevel.DEBUG, logger,
						// simple("${body}").toString())
						.setHeader("id", xpath("//*[local-name()='identifier']/text()").stringResult())
						.setHeader("oid", xpath("//*[local-name()='oid']/text()").stringResult())
						.setHeader("name", xpath("//*[local-name()='name']/text()").stringResult())
						.end().process(new Processor()
						{
							@Override
							public void process(Exchange exchange) throws Exception
							{
								Message m = exchange.getIn();
								Object b = m.getBody();
								if (b != null && b instanceof List)
								{
									ArrayList<SiteModel> sites = (ArrayList<SiteModel>) b;
									for (Object element : sites)
									{
										SiteModel siteModel = (SiteModel) element;
										logger.debug("### Site -> oid={}, identifier={}, name={}", siteModel.getOid(), siteModel.getId(), siteModel.getName());
									}
								} else
								{
									logger.error("No SiteModel");
								}
							}
						});
			}

			@Override
			protected void checkInitialized() throws Exception
			{
				PropertiesComponent c = (PropertiesComponent) getContext().getComponent("properties");
				// c.setLocation("classpath:camel.properties");
				c.addFunction(new PropertiesDbResolveFunction());

				super.checkInitialized();
			}
		});
		main.start();
		main.run(new String[] {});
	}

}